SystemOrganization addCategory: #NonInteractiveTranscript!

Object subclass: #NonInteractiveTranscript
	instanceVariableNames: 'stream fileName accessSemaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NonInteractiveTranscript'!

!NonInteractiveTranscript commentStamp: '<historical>' prior: 0!
I am NonInteractiveTranscript, a replacement for Transcript, writing everything to a file or stdout.

This is useful when running headless.

	NonInteractiveTranscript file install.
	
To connect to the output stream of the virtual machine process choose stdout.

	NonInteractiveTranscript stdout install.
!

!NonInteractiveTranscript class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 9/30/2011 10:29'!
defaultLogFileName
	"The default file name that will be used to write to.
	This should probably become a system setting."
	
	^ 'PharoTranscript.log'! !

!NonInteractiveTranscript class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/4/2011 17:34'!
file
	"Create a new NonInteractiveTranscript that will output 
	to the default file named by #defaultLogFileName."
	
	^ self onFileNamed: self defaultLogFileName ! !

!NonInteractiveTranscript class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 7/6/2012 10:46'!
initialize
	"Make sure to cleanup on shutdown"
	
	Smalltalk addToShutDownList: self	! !

!NonInteractiveTranscript class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 7/6/2012 10:43'!
isInstalled
	^ Transcript class = self! !

!NonInteractiveTranscript class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/4/2011 17:33'!
onFileNamed: fileName
	"Create a NonInteractiveTranscript that will write to fileName."
	
	^ self new
		fileName: fileName;
		yourself! !

!NonInteractiveTranscript class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 7/6/2012 10:54'!
shutDown
	"Send close to all our instances.
	Their streams will be reopened on first use later on."
	
	self allInstances do: [ :each | each close ]! !

!NonInteractiveTranscript class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/4/2011 17:33'!
stdout
	"Create a new NonInteractiveTranscript that will output 
	to the special standard output stream of the virtual machine process."
	
	^ self onFileNamed: #stdout! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 19:30'!
<< anObject
	self show: anObject! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 19:29'!
clear! !

!NonInteractiveTranscript methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/6/2012 10:38'!
close
	self critical: [ 
		stream ifNotNil: [ 
			self isStdout 
				ifTrue: [ stream flush ]
				ifFalse: [ stream close ].
			stream := nil ] ]! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/30/2011 10:18'!
cr 
	self critical: [ 
		self stream cr ]! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 20:02'!
crShow: anObject
	self critical: [
		self
			cr; 
			show: anObject ]! !

!NonInteractiveTranscript methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/28/2011 19:55'!
critical: block
	^ accessSemaphore 
		critical: block ! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 19:48'!
endEntry
	self flush! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 19:30'!
ensureCr! !

!NonInteractiveTranscript methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/4/2011 17:49'!
fileName
	"The file name that I will write to.
	Lazy initialized to a default.
	When I connect to the standard output stream of the 
	virtual machine process I return #stdout."
	
	^ fileName ifNil: [ fileName := self class defaultLogFileName ]! !

!NonInteractiveTranscript methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/6/2012 10:42'!
fileName: anObject
	"Initialize me to write to a file stream described by anObject.
	anObject is either a String naming a file or the special #stdout value
	for using the standard output stream of the virtual machine process."
	
	self close.
	fileName := anObject! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/30/2011 10:18'!
flush
	self critical: [ 
		self stream flush ]! !

!NonInteractiveTranscript methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/28/2011 21:22'!
initialize
	super initialize.
	accessSemaphore := Mutex new! !

!NonInteractiveTranscript methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/4/2011 18:03'!
initializeStream
	"Open the file stream that I write to or connect to #stdout.
	I use the proper line end convention.
	I will append to regular files."
	
	stream := self isStdout 
		ifTrue: [ FileStream stdout ]
		ifFalse: [ FileStream fileNamed: self fileName ].
	stream wantsLineEndConversion: true.
	self isStdout
		ifTrue: [ stream converter "Force lazy initialization of converter" ] 
		ifFalse: [ stream setToEnd ].
	^ stream
	! !

!NonInteractiveTranscript methodsFor: 'installation' stamp: 'SvenVanCaekenberghe 12/4/2011 17:31'!
install
	"Install me as a replacement for Transcript"
	
	Smalltalk globals 
		at: #Transcript 
		put: self! !

!NonInteractiveTranscript methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/28/2011 19:26'!
isSelfEvaluating
	self == Transcript ifTrue: [ ^ true ].
	^ super isSelfEvaluating! !

!NonInteractiveTranscript methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2011 17:46'!
isStdout
	"Answer true when I am setup to be connected to the special
	standard output stream of the virtual machine process."
	
	^ fileName = #stdout! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/30/2011 10:18'!
nextPut: aCharacter
	^ self critical: [ 
		self stream nextPut: aCharacter ]! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/30/2011 10:19'!
nextPutAll: aCollection
	^ self critical: [ 
		self stream nextPutAll: aCollection ]! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 21:23'!
print: anObject
	self nextPutAll: anObject asString! !

!NonInteractiveTranscript methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/28/2011 19:26'!
printOn: aStream
	self == Transcript ifFalse: [ ^ super printOn: aStream ].
	aStream nextPutAll: 'Transcript'! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 19:29'!
reset! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2011 20:02'!
show: anObject
	self critical: [
		self 
			print: anObject; 
			endEntry ]! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/30/2011 10:19'!
space 
	^ self critical: [ 
		self stream space ]! !

!NonInteractiveTranscript methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2011 10:32'!
stream
	"The file stream I am writing to.
	Lazy initialized so as not to create the file when it is not used."
	
	^ stream ifNil: [ self initializeStream ]! !

!NonInteractiveTranscript methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/30/2011 10:19'!
tab 
	^ self critical: [ 
		self stream tab ]! !
NonInteractiveTranscript initialize!
